@@ -22,10 +22,7 @@ impl Factory {
     pub fn from_scons_vars(scons_vars: &pio::project::SconsVariables) -> Result<Self> {
         let clang_args = cli::NativeCommandArgs::new(&scons_vars.incflags)
             .chain(cli::NativeCommandArgs::new(
-                scons_vars
-                    .clangargs
-                    .as_deref()
-                    .unwrap_or_default(),
+                scons_vars.clangargs.as_deref().unwrap_or_default(),
             ))
             .collect();
 
@@ -103,12 +100,14 @@ impl Factory {
             .layout_tests(false)
             .rustfmt_bindings(false)
             .derive_default(true)
-            //.ctypes_prefix(c_types)
             .clang_arg("-D__bindgen")
+            // Include directories provided by the build system
+            // should be first on the search path (before sysroot includes),
+            // or else libc's <dirent.h> does not correctly override sysroot's <dirent.h>
+            .clang_args(&self.clang_args)
             .clang_args(sysroot_args)
             .clang_args(&["-x", if cpp { "c++" } else { "c" }])
-            .clang_args(cpp_args)
-            .clang_args(&self.clang_args);
+            .clang_args(cpp_args);
 
         log::debug!(
             "Bindgen builder factory flags: {:?}",
@@ -119,12 +118,13 @@ impl Factory {
     }
 }
 
-pub fn run(builder: bindgen::Builder) -> Result<()> {
+pub fn run(builder: bindgen::Builder) -> Result<PathBuf> {
     let output_file = PathBuf::from(env::var("OUT_DIR")?).join("bindings.rs");
     run_for_file(builder, &output_file)?;
 
     cargo::set_rustc_env(VAR_BINDINGS_FILE, output_file.try_to_str()?);
-    Ok(())
+
+    Ok(output_file)
 }
 
 pub fn run_for_file(builder: bindgen::Builder, output_file: impl AsRef<Path>) -> Result<()> {
